<?php

/*
 +--------------------------------------------------------------------+
 | CiviCRM version 1.9                                                |
 +--------------------------------------------------------------------+
 | Copyright CiviCRM LLC (c) 2004-2007                                |
 +--------------------------------------------------------------------+
 | This file is a part of CiviCRM.                                    |
 |                                                                    |
 | CiviCRM is free software; you can copy, modify, and distribute it  |
 | under the terms of the Affero General Public License Version 1,    |
 | March 2002.                                                        |
 |                                                                    |
 | CiviCRM is distributed in the hope that it will be useful, but     |
 | WITHOUT ANY WARRANTY; without even the implied warranty of         |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.               |
 | See the Affero General Public License for more details.            |
 |                                                                    |
 | You should have received a copy of the Affero General Public       |
 | License along with this program; if not, contact CiviCRM LLC       |
 | at info[AT]civicrm[DOT]org.  If you have questions about the       |
 | Affero General Public License or the licensing  of CiviCRM,        |
 | see the CiviCRM license FAQ at http://civicrm.org/licensing        |
 +--------------------------------------------------------------------+
*/

/**
 *
 * @package CRM
 * @copyright CiviCRM LLC (c) 2004-2007
 * $Id: AJAX.php 11623 2007-09-27 14:04:18Z kurund $
 *
 */

require_once 'CRM/Core/Page.php';

/**
 * This class contains all the function that are called using AJAX (dojo)
 */
class CRM_Core_Page_AJAX extends CRM_Core_Page 
{
    /**
     * Run the page
     */
    function run( &$args ) 
    {
        $this->invoke( $args );
        exit( );
    }

    /**
     * Invoke function that redirects to respective functions
     */
    function invoke( &$args ) 
    {
        // intialize the system
        $config =& CRM_Core_Config::singleton( );

        if ( $args[0] != 'civicrm' && $args[1] != 'ajax' ) {
            exit( );
        }

        switch ( $args[2] ) {

        case 'help':
            return $this->help( $config );

        case 'search':
            return $this->search( $config );

        case 'state':
            return $this->state( $config );

        case 'country':
            return $this->country( $config );

        case 'status':
            return $this->status( $config );

        case 'event':
            return $this->event( $config );
        
        case 'eventType':
            return $this->eventType( $config );

        case 'message':
            return $this->message( $config );

        case 'template':
            return $this->template( $config );

        default:
            return;
        }
    }

    /**
     * Function to get help messages
     */
    function help( &$config ) 
    {
        $id   = urldecode( $_GET['id'] );
        $file = urldecode( $_GET['file'] );

        $template =& CRM_Core_Smarty::singleton( );
        $file = str_replace( '.tpl', '.hlp', $file );

        $template->assign( 'id', $id );
        echo $template->fetch( $file );
    }

    /**
     * Function for building contact combo box
     */
    function search( &$config ) 
    {
        require_once 'CRM/Utils/Type.php';
        $domainID  = CRM_Utils_Type::escape( $_GET['d'], 'Integer' );
        $name      = strtolower( CRM_Utils_Type::escape( $_GET['s'], 'String'  ) );
        
        $shared = null;
        if ( isset($_GET['sh']) ) {
            $shared = CRM_Utils_Type::escape( $_GET['sh'], 'Integer');
        }

        if ( $shared ) {
            $query = "
SELECT CONCAT_WS( ', ', household_name, LEFT( street_address, 25 ) , city ) 'sort_name', 
civicrm_household.contact_id 'id'
FROM civicrm_household
LEFT JOIN civicrm_location ON civicrm_location.entity_id=civicrm_household.contact_id 
AND civicrm_location.is_primary=1 
AND civicrm_location.entity_table='civicrm_contact'
LEFT JOIN civicrm_address ON civicrm_address.location_id=civicrm_location.id
where household_name LIKE '$name%'
ORDER BY household_name LIMIT 6";
        } else {
            $query = "
SELECT sort_name, id
FROM civicrm_contact
WHERE sort_name LIKE '$name%'
AND domain_id = $domainID
ORDER BY sort_name LIMIT 6";
        }

        $nullArray = array( );
        $dao = CRM_Core_DAO::executeQuery( $query, $nullArray );

        $count = 0;
        $elements = array( );
        while ( $dao->fetch( ) && $count < 5 ) {
            $elements[] = array( $dao->sort_name, $dao->id );
            $count++;
        }
        
        require_once 'Services/JSON.php';
        $json =& new Services_JSON( );
        echo $json->encode( $elements );
    }

    /**
     * Function for building Event combo box
     */
    function event( &$config ) 
    {
        require_once 'CRM/Utils/Type.php';
        $domainID = CRM_Utils_Type::escape( $_GET['d'], 'Integer' );
        $name     = strtolower( CRM_Utils_Type::escape( $_GET['s'], 'String'  ) );

        $query = "
SELECT title
  FROM civicrm_event
 WHERE domain_id = $domainID
   AND title LIKE '$name%'
ORDER BY title
LIMIT 6";

        $nullArray = array( );
        $dao = CRM_Core_DAO::executeQuery( $query, $nullArray );

        $count = 0;
        $elements = array( );
        while ( $dao->fetch( ) && $count < 5 ) {
            $elements[] = array( $dao->title, $dao->title );
            $count++;
        }

        require_once 'Services/JSON.php';
        $json =& new Services_JSON( );
        echo $json->encode( $elements );
    }

    /**
     * Function for building Event Type combo box
     */
    function eventType( &$config ) 
    {
        require_once 'CRM/Utils/Type.php';
        $domainID = CRM_Utils_Type::escape( $_GET['d'], 'Integer' );
        $name     = strtolower( CRM_Utils_Type::escape( $_GET['s'], 'String'  ) );

        $query ="
SELECT v.label 
FROM   civicrm_option_value v,
       civicrm_option_group g
WHERE  v.option_group_id = g.id 
AND g.name = 'event_type'
AND v.is_active = 1
AND v.label  LIKE '$name%' 
ORDER by v.weight LIMIT 5";

        $nullArray = array( );
        $dao = CRM_Core_DAO::executeQuery( $query, $nullArray );

        $count = 0;
        $elements = array( );
        while ( $dao->fetch( ) && $count < 5 ) {
            $elements[] = array( $dao->label, $dao->label );
            $count++;
        }

        require_once 'Services/JSON.php';
        $json =& new Services_JSON( );
        echo $json->encode( $elements );
    }

    /**
     * Function to show import status
     */
    function status( &$config ) 
    {
        // make sure we get an id
        if ( ! isset( $_GET['id'] ) ) {
            return;
        }

        $file = "{$config->uploadDir}status_{$_GET['id']}.txt";
        if ( file_exists( $file ) ) {
            $str = file_get_contents( $file );
            echo $str;
        } else {
            require_once 'Services/JSON.php';
            $json =& new Services_JSON( );
            $status = "<div class='description'>&nbsp; " . ts('No processing status reported yet.') . "</div>";
            echo $json->encode( array( 0, $status ) );
        }
    }

    /**
     * Function to build message template combo box
     */
    function message( &$config ) 
    {
        require_once 'CRM/Utils/Type.php';
        $domainID = CRM_Utils_Type::escape( $_GET['d'], 'Integer' );

        $query = "
SELECT id, msg_title,msg_text,msg_subject,msg_html
  FROM civicrm_msg_template
 WHERE domain_id = $domainID
 AND is_active =1 
ORDER BY msg_title
LIMIT 6";

        $nullArray = array( );
        $dao = CRM_Core_DAO::executeQuery( $query, $nullArray );

        $count = 0;
        $elements = array( );
        while ( $dao->fetch( ) && $count < 5 ) {
            $elements[] = array( $dao->msg_title,
                                 $dao->msg_text . "^A" . $dao->msg_subject. "^A" .$dao->msg_html,
                                 $dao->id );
            $count++;
        }
        require_once 'Services/JSON.php';
        $json =& new Services_JSON( );
        echo $json->encode( $elements );
    }

    /**
     * Function to build state province combo box
     */
    function state( &$config ) 
    {
        require_once 'CRM/Utils/Type.php';
        $countryName  = $stateName = null;
        if ( isset( $_GET['node'] ) ) {
            $countryName  = CRM_Utils_Type::escape( $_GET['node'], 'String');
        }

        if ( isset( $_GET['s'] ) ) {
            $stateName    = trim (CRM_Utils_Type::escape( $_GET['s']   , 'String') );
        }

        //temporary fix to handle locales other than default US,
        //trying to emulate anti ts() function, will make it cleaner
        //in v2.0
        if ( $config->lcMessages != 'en_US') {
            //get all the countries
            $query = "
SELECT id, name
  FROM civicrm_country
ORDER BY name";

            $nullArray = array( );
            $dao = CRM_Core_DAO::executeQuery( $query, $nullArray );
            
            $countries = array( );
            while ( $dao->fetch( ) ) {
                $countries[$dao->name] = ts($dao->name);
            }

            // get the country name in en_US, since db has this locale
            $countryName = array_search( $countryName, $countries );
        }

        $query = "
SELECT civicrm_state_province.name name, civicrm_state_province.id id
  FROM civicrm_state_province, civicrm_country
 WHERE civicrm_state_province.country_id = civicrm_country.id
  AND  LOWER(civicrm_country.name) = LOWER('$countryName')";

        if ( isset( $_GET['s'] ) ) {
            $query .= " AND  civicrm_state_province.name LIKE LOWER('$stateName%') ";
        }

        $query .= " ORDER BY name";

        $nullArray = array( );
        $dao = CRM_Core_DAO::executeQuery( $query, $nullArray );

        $count = 0;
        $elements = array( );
        while ( $dao->fetch( ) && $count < 5 ) {
            $elements[] = array( ts($dao->name), $dao->id );
            $count++;
        }

        if ( empty( $elements ) ) {
            $elements[] = array( $stateName, $stateName );
        }

        require_once 'Services/JSON.php';
        $json =& new Services_JSON( );
        echo $json->encode( $elements );
    }

    /**
     * Function to build country combo box
     */
    function country( &$config ) 
    {
        //get the country limit and restrict the combo select options
        $limitCodes = $config->countryLimit( );
        if ( ! is_array( $limitCodes ) ) {
            $limitCodes = array( $config->countryLimit => 1);
        }
        
        $limitCodes = array_intersect( CRM_Core_PseudoConstant::countryIsoCode(), $limitCodes);
        if ( count($limitCodes) ) {
            $whereClause = " iso_code IN ('" . implode("', '", $limitCodes) . "')";
        } else {
            $whereClause = " 1";
        }

        require_once 'CRM/Utils/Type.php';
        $name     = CRM_Utils_Type::escape( $_GET['s'], 'String'  );

        $query = "
SELECT id, name
  FROM civicrm_country
 WHERE civicrm_country.name LIKE LOWER('$name%')
   AND {$whereClause} 
ORDER BY name";

        $nullArray = array( );
        $dao = CRM_Core_DAO::executeQuery( $query, $nullArray );

        $count = 0;
        $elements = array( );
        while ( $dao->fetch( ) && $count < 5 ) {
            $elements[] = array( ts($dao->name), $dao->id );
            $count++;
        }

        if ( empty( $elements ) ) {
            $elements[] = array( $name, $name );
        }

        require_once 'Services/JSON.php';
        $json =& new Services_JSON( );
        echo $json->encode( $elements );
    }

    /**
     * Function to fetch the template text/html messages
     */
    function template( &$config ) 
    {
        require_once 'CRM/Utils/Type.php';
        $templateId = CRM_Utils_Type::escape( $_GET['tid'], 'Integer' );

        require_once "CRM/Core/DAO/MessageTemplates.php";
        $messageTemplate =& new CRM_Core_DAO_MessageTemplates( );
        $messageTemplate->id = $templateId;
        $messageTemplate->selectAdd( );
        $messageTemplate->selectAdd( 'msg_text, msg_html' );
        $messageTemplate->find( true );
        
        $elements = array( $messageTemplate->msg_text, $messageTemplate->msg_html );

        require_once 'Services/JSON.php';
        $json =& new Services_JSON( );
        echo $json->encode( $elements );
    }


}

?>